// Copyright (C) 2018 Avatao.com Innovative Learning Kft.
// All Rights Reserved. See LICENSE file for details.

import { Injectable } from '@angular/core';
import { QueueingSubject } from './queueing-subject';
import { Observable } from 'rxjs';
import websocketConnect from 'rxjs-websockets';
import { filter, map, share } from 'rxjs/operators';
import { WebSocketMessage } from '../message-types/websocket-message';


function jsonWebsocketConnect(url: string, input: Observable<object>, protocols?: string | string[]) {
  const jsonInput = input.pipe(map(message => JSON.stringify(message)));
  const { connectionStatus, messages } = websocketConnect(url, jsonInput, protocols);
  const jsonMessages = messages.pipe(map(message => JSON.parse(message)));
  return { connectionStatus, messages: jsonMessages };
}

@Injectable()
export class WebSocketService {
  private uplink: QueueingSubject<object>;
  public downlink: Observable<WebSocketMessage<undefined>>;

  constructor() {}

  public connect() {
    if (this.downlink) {
      return;
    }

    const wsproto = (location.protocol === 'https:') ? 'wss://' : 'ws://';
    this.downlink = jsonWebsocketConnect(
      wsproto + window.location.host + '/ws',
      this.uplink = new QueueingSubject<object>()
    ).messages.pipe(
        map(message => <WebSocketMessage<undefined>> message),
        share()
      );
    console.log('ws connected');
  }

  public observeKey<T>(key: string): Observable<WebSocketMessage<T>> {
    return this.downlink.pipe(filter(message => message.key === key));
  }

  public send(key: string, data?: any, trigger?: any): void {
    const message = {'key': key};
    if (data) { message['data'] = data; }
    if (trigger) { message['trigger'] = trigger; }
    this.sendJSON(message);
  }

  public sendJSON(json: any) {
    this.uplink.next(json);
  }
}
